using System;

namespace Rsdn.SmartApp.Demos
{
	internal class Program
	{
		private static void Main()
		{
			ServiceManager rootMgr = new ServiceManager();

			// Publish service
			rootMgr.Publish<IService>(new ServiceImpl("Root service"));
			Print(rootMgr);

			// Cascade managers
			ServiceManager cascadedMgr = new ServiceManager(rootMgr);
			Print(cascadedMgr);

			// Override service implementation
			IServiceCookie cookie = cascadedMgr.Publish<IService>(
				new ServiceImpl("Cascaded service"));
			Print(cascadedMgr);

			// Remove service implementation
			cascadedMgr.Unpublish(cookie);
			Print(cascadedMgr);

			// Automatic service publishing
			ExtensionManager extMgr = new ExtensionManager(null);
			extMgr.Scan(typeof (Program).Assembly.GetTypes(),
				new ServicePublishingStrategy(cascadedMgr));
			Print(cascadedMgr);

			Console.Read();
		}

		private static void Print(IServiceProvider provider)
		{
			provider.GetService<IService>().PrintName(Console.Out);
		}
	}
}
