using System;
using System.Collections.Generic;
using System.Collections;

namespace Rsdn.SmartApp
{
	/// <summary>
	///     .
	/// </summary>
	public static class CollectionUtils
	{
		/// <summary>
		///  <see cref="IEnumerable"/>    <see cref="IEnumerable{T}"/>
		/// </summary>
		/// <remarks>
		/// ,  null     .
		/// </remarks>
		public static IEnumerable<T> Convert<T>(IEnumerable oldStyleSrc) where T : class
		{
			foreach (object o in oldStyleSrc)
			{
				T elem = o as T;
				if (elem != null)
					yield return elem;
			}
		}

		/// <summary>
		///  <see cref="ICollection"/>    .
		/// </summary>
		public static T[] ToArray<T>(this ICollection collection)
		{
			T[] res = new T[collection.Count];
			int i = 0;
			foreach (T elem in collection)
			{
				res[i] = elem;
				i++;
			}
			return res;
		}

		/// <summary>
		///  <see cref="ICollection{T}"/>    .
		/// </summary>
		public static T[] ToArray<T>(this ICollection<T> collection)
		{
			T[] res = new T[collection.Count];
			int i = 0;
			foreach (T elem in collection)
			{
				res[i] = elem;
				i++;
			}
			return res;
		}

		/// <summary>
		///  .
		/// </summary>
		public static IEnumerable<T> Filter<T>(IEnumerable<T> enumerable, Predicate<T> leave)
		{
			foreach (T elem in enumerable)
				if (leave(elem))
					yield return elem;
		}

		/// <summary>
		///  <see cref="ICollection"/>       
		///   <see cref="IComparer{T}"/>.
		/// </summary>
		public static T[] ToArray<T>(ICollection collection, IComparer<T> comparer)
		{
			if (comparer == null)
				throw new ArgumentNullException("comparer");
			T[] res = ToArray<T>(collection);
			Array.Sort(res, comparer);
			return res;
		}

		/// <summary>
		///  <see cref="ICollection"/>       
		///  .
		/// </summary>
		public static T[] ToArray<T>(ICollection collection, Comparison<T> comparison)
		{
			if (comparison == null)
				throw new ArgumentNullException("comparison");
			T[] res = ToArray<T>(collection);
			Array.Sort(res, comparison);
			return res;
		}

		/// <summary>
		///  <see cref="ICollection{T}"/>       
		///  .
		/// </summary>
		public static T[] ToArray<T>(ICollection<T> collection, Comparison<T> comparison)
		{
			if (comparison == null)
				throw new ArgumentNullException("comparison");
			T[] res = ToArray(collection);
			Array.Sort(res, comparison);
			return res;
		}

		/// <summary>
		///  <see cref="IEnumerable"/>    .
		/// </summary>
		public static T[] ToArray<T>(IEnumerable collection) where T : class
		{
			List<T> res = new List<T>(Convert<T>(collection));
			return res.ToArray();
		}
		
		/// <summary>
		///          .
		/// </summary>
		public static TOut[] ConvertAll<TIn, TOut>(this ICollection<TIn> collection,
			Converter<TIn, TOut> converter)
		{
			TOut[] res = new TOut[collection.Count];
			int i = 0;
			foreach (TIn elem in collection)
			{
				res[i] = converter(elem);
				i++;
			}
			return res;
		}

		/// <summary>
		/// ,      .
		/// </summary>
		public delegate TKey GetKey<TKey, TValue>(TValue source);
		
		/// <summary>
		///  generic-  .
		/// </summary>
		public static IDictionary<TKey, TValue> Convert2Dictionary<TKey, TValue>(
			this IEnumerable<TValue> source, GetKey<TKey, TValue> keyGetter)
		{
			Dictionary<TKey, TValue> res = new Dictionary<TKey, TValue>();
			foreach (TValue val in source)
				res.Add(keyGetter(val), val);
			return res;
		}

		/// <summary>
		///    .
		/// </summary>
		public static IDictionary<TKey, TValue> Convert2Dictionary<TKey, TValue>(
			this IEnumerable source, GetKey<TKey, TValue> keyGetter)
		{
			Dictionary<TKey, TValue> res = new Dictionary<TKey, TValue>();
			foreach (TValue val in source)
				res.Add(keyGetter(val), val);
			return res;
		}
		
		/// <summary>
		///    .
		/// </summary>
		public static Comparison<T> ReverseComparision<T>(Comparison<T> srcComparision)
		{
			return delegate(T e1, T e2) {
				int cr = srcComparision(e1, e2);
				return cr > 0 ? -1 : cr < 0 ? 1 : 0;
			};
		}

		/// <summary>
		///       .
		/// </summary>
		public static void ForEach<T>(this IEnumerable<T> src, Action<T> action)
		{
			foreach (T element in src)
				action(element);
		}

		/// <summary>
		///      .
		/// </summary>
		public static T[] Join<T>(params IEnumerable<T>[] srcs)
		{
			if (srcs.Length == 0)
				return new T[0];
			List<T> res = new List<T>(srcs[0]);
			for (int i = 1; i < srcs.Length; i++)
				res.AddRange(srcs[i]);
			return res.ToArray();
		}
	}
}
