using System;
using System.Xml;
using System.Xml.Linq;
using System.Xml.Schema;
using System.Xml.Serialization;

namespace Rsdn.SmartApp.Configuration
{
	internal class XmlSectionSerializer : IConfigSectionSerializer
	{
		private static readonly ElementsCache<Type, XmlSerializer> _xmlSerializers =
			new ElementsCache<Type, XmlSerializer>(
				delegate(Type type) { return new XmlSerializer(type); });

		private readonly Type _contractType;
		private readonly XmlSchema _schema;

		public XmlSectionSerializer(Type contractType, XmlSchema schema)
		{
			_contractType = contractType;
			_schema = schema;
		}

		/// <summary>
		///  .
		/// </summary>
		public object Deserialize(XElement node)
		{
			var xs = _xmlSerializers.Get(_contractType);
			var reader = node.CreateReader();
			if (_schema != null)
			{
				var rdrSettings = new XmlReaderSettings();
				rdrSettings.Schemas.Add(_schema);
				rdrSettings.ValidationType = ValidationType.Schema;
				reader = XmlReader.Create(reader, rdrSettings);
			}
			return xs.Deserialize(reader);
		}

		/// <summary>
		///    .
		/// </summary>
		public object CreateDefaultSection()
		{
			return Activator.CreateInstance(_contractType);
		}
	}
}
