using System;
using System.Collections.Generic;
using System.Reflection;

namespace Rsdn.SmartApp
{
	/// <summary>
	///   <see cref="ITypeSchemaScanner"/>
	/// </summary>
	public class DefaultScanner : ITypeSchemaScanner
	{
		/// <summary>
		///   .
		/// </summary>
		public ITypeSchema GetTypeSchema(Type type)
		{
			if (type == null)
				throw new ArgumentNullException("type");
			TypeSchemaAttribute[] attrs = (TypeSchemaAttribute[])type
				.GetCustomAttributes(typeof (TypeSchemaAttribute), false);
			if (attrs.Length == 0)
				throw new ApplicationException("Type must be marked by "
					+ "Rsdn.SmartApp.TestSchemaAttribute");
			if (attrs.Length != 1)
				throw new ApplicationException("Only one instance of "
					+ "Rsdn.SmartApp.TestSchemaAttribute is allowed");
			return new TypeSchema(type.FullName, attrs[0].Id, ScanProperties(type));
		}

		private SchemaProperty[] ScanProperties(Type type)
		{
			List<SchemaProperty> props = new List<SchemaProperty>();
			foreach (PropertyInfo pi in type.GetProperties())
			{
				
				SchemaPropertyAttribute[] attrs = (SchemaPropertyAttribute[])pi
					.GetCustomAttributes(typeof (SchemaPropertyAttribute), false);
				if (attrs.Length > 1)
					throw new ApplicationException("Only one instance of "
					+ "Rsdn.SmartApp.SchemaPropertyAttribute is allowed");
				if (attrs.Length == 1)
				{
					if (!(pi.CanRead && pi.CanWrite))
						throw new ApplicationException(
							"Schema property must be read/write property");
					props.Add(new SchemaProperty(pi.Name, attrs[0].Id));
				}
			}
			return props.ToArray();
		}
	}
}
