using System;
using System.Collections.Generic;
using System.Threading;

namespace Rsdn.SmartApp
{
	/// <summary>
	///     .
	/// </summary>
	public delegate E CreateElement<K, E>(K key);

	/// <summary>
	///  .    .
	/// </summary>
	public class ElementsCache<K, E>
	{
		private readonly CreateElement<K, E> _elementCreator;
		private readonly Dictionary<K, E> _elements = new Dictionary<K, E>();
		private readonly ReaderWriterLockSlim m_CacheLock = new ReaderWriterLockSlim();

		/// <summary>
		///  .
		/// </summary>
		public ElementsCache(CreateElement<K, E> elementCreator)
		{
			_elementCreator = elementCreator;
		}

		/// <summary>
		///  .
		/// </summary>
		public CreateElement<K, E> ElementCreator
		{
			get { return _elementCreator; }
		}

		/// <summary>
		///   .
		/// </summary>
		public ICollection<K> Keys
		{
			get
			{
				using (m_CacheLock.GetReaderLock())
				{
					var keys = new List<K>(_elements.Keys);
					return keys;
				}
			}
		}

		/// <summary>
		///    .
		/// </summary>
		public E Get(K key)
		{
			if (key == null)
				throw new ArgumentNullException("key");
			using (m_CacheLock.GetUpgradableReaderLock())
			{
				E element;
				if (!_elements.TryGetValue(key, out element))
					using (m_CacheLock.GetWriterLock())
					{
						element = _elementCreator(key);
						_elements.Add(key, element);
						OnAfterElementCreated(key, element);
					}
				return element;
			}
		}

		/// <summary>
		///     .
		/// </summary>
		protected virtual void OnAfterElementCreated(K key, E element)
		{}

		/// <summary>
		///    .
		/// </summary>
		public bool TryGet(K key, out E element)
		{
			if (key == null)
				throw new ArgumentNullException("key");
			using (m_CacheLock.GetReaderLock())
				return _elements.TryGetValue(key, out element);
		}

		/// <summary>
		/// ,       .
		/// </summary>
		public bool Contains(K key)
		{
			if (key == null)
				throw new ArgumentNullException("key");
			using (m_CacheLock.GetReaderLock())
				return _elements.ContainsKey(key);
		}

		/// <summary>
		///      .
		/// </summary>
		public virtual void Drop(K key)
		{
			if (key == null)
				throw new ArgumentNullException("key");
			using (m_CacheLock.GetWriterLock())
				_elements.Remove(key);
		}

		/// <summary>
		///  .
		/// </summary>
		public virtual void Reset()
		{
			using (m_CacheLock.GetWriterLock())
				_elements.Clear();
		}
	}
}
