using System;
using System.Collections.Generic;

namespace Rsdn.SmartApp
{
	/// <summary>
	///    .
	/// </summary>
	public class ExtensionManager : IExtensionManager, IServiceProvider
	{
		private readonly ServiceManager _serviceManager;
		private static readonly Attribute[] _emptyAttrs = new Attribute[0];

		private static Attribute[] ExtractTypeAttributes(Type type,
			IExtensionAttachmentContext context)
		{
			object[] attrs = type.GetCustomAttributes(true);
			List<IScanFilterAttribute> filters = new List<IScanFilterAttribute>();

			foreach (object attr in attrs)
			{
				if (attr is DisableAllScanAttribute)
					return _emptyAttrs;
				IScanFilterAttribute sfa = attr as IScanFilterAttribute;
				if (sfa != null)
					filters.Add(sfa);
			}

			List<Attribute> res = new List<Attribute>();
			foreach (object attr in attrs)
			{
				Attribute a = attr as Attribute;
				if (a != null)
				{
					bool filtered = false;
					foreach (IScanFilterAttribute sfa in filters)
						if (!sfa.AllowAttach(context, a))
						{
							filtered = true;
							break;
						}
					if (!filtered)
						res.Add(a);
				}
			}
			return res.ToArray();
		}

		/// <summary>
		///  .
		/// </summary>
		public ExtensionManager(IServiceProvider serviceProvider)
		{
			_serviceManager = serviceProvider == null
				? new ServiceManager()
				: new ServiceManager(serviceProvider);
			_serviceManager.Publish<IExtensionManager>(this);
		}

		/// <summary>
		///  .
		/// </summary>
		public void Scan(Type[] types, IExtensionAttachmentStrategy strategy)
		{
			if (types == null)
				throw new ArgumentNullException("types");
			if (strategy == null)
				throw new ArgumentNullException("strategy");
			foreach (Type t in types)
			{
				ExtensionAttachmentContext context = new ExtensionAttachmentContext(this, t);
				foreach (Attribute attr in ExtractTypeAttributes(t, context))
					strategy.Attach(context, attr);
			}
		}

		/// <summary>
		///  ,   T
		/// </summary>
		public T GetService<T>() where T : class
		{
			return _serviceManager.GetService<T>();
		}
	}
}
