using System;

namespace Rsdn.SmartApp
{
	using FactoriesSvc = IRegKeyedElementsService<Type, ExtensionStrategyFactoryInfo>;

	/// <summary>
	///    .
	/// </summary>
	public class StrategyFactoryStrategy :
		RegKeyedElementsStrategy<Type, ExtensionStrategyFactoryInfo,
			ExtensionStrategyFactoryAttribute>
	{
		private static readonly ElementsCache<Type, IExtensionStrategyFactory> _factories =
			new ElementsCache<Type, IExtensionStrategyFactory>(
				delegate (Type type)
				{
					return (IExtensionStrategyFactory)Activator.CreateInstance(type);
				});

		/// <summary>
		///  .
		/// </summary>
		public StrategyFactoryStrategy(IServicePublisher publisher) : base(publisher)
		{}

		/// <summary>
		///  .
		/// </summary>
		public override ExtensionStrategyFactoryInfo CreateElement(IExtensionAttachmentContext context,
			ExtensionStrategyFactoryAttribute attr)
		{
			if (!typeof(IExtensionStrategyFactory).IsAssignableFrom(
					context.ExtensionType))
				throw new InvalidExtensionTypeException("Strategy factory must implement"
					+ " 'Rsdn.SmartApp.IExtensionStrategyFactory' interface");
			return new ExtensionStrategyFactoryInfo(context.ExtensionType);
		}

		/// <summary>
		///      .
		/// </summary>
		public static void ScanWithFactory(Type factoryType,
			IServiceProvider provider,
			IExtensionManager extensionManager,
			Type[] types)
		{
			FactoriesSvc svc = provider.GetService<FactoriesSvc>();
			ScanWithFactory(provider, extensionManager, svc.GetElement(factoryType), types);
		}

		/// <summary>
		///    .
		/// </summary>
		public static void ScanWithAllFactories(
			IServiceProvider provider,
			IExtensionManager extensionManager,
			Type[] types)
		{
			FactoriesSvc svc = provider.GetService<FactoriesSvc>();
			foreach (ExtensionStrategyFactoryInfo factoryInfo in svc.GetRegisteredElements())
				ScanWithFactory(provider, extensionManager, factoryInfo, types);
		}

		/// <summary>
		///        .
		/// </summary>
		public static void RegisterAndScan(
			IServicePublisher publisher,
			IExtensionManager extensionManager,
			Type[] types)
		{
			extensionManager.Scan(types, new StrategyFactoryStrategy(publisher));
			ScanWithAllFactories(publisher, extensionManager, types);
		}

		private static void ScanWithFactory(IServiceProvider provider,
			IExtensionManager extManager,
			ExtensionStrategyFactoryInfo factoryInfo,
			Type[] types)
		{
			IExtensionStrategyFactory factory = _factories.Get(factoryInfo.Type);
			foreach (IExtensionAttachmentStrategy strat in factory.CreateStrategies(provider))
				extManager.Scan(types, strat);
		}
	}
}
