using System;
using System.IO;
using System.Reflection;
using System.Threading;

using NUnit.Framework;

namespace Rsdn.SmartApp.Configuration
{
	[TestFixture]
	public class FileConfigDataProviderTest
	{
		private static readonly string _exeDir =
			Path.GetDirectoryName(
				new Uri(Assembly.GetExecutingAssembly().CodeBase).AbsolutePath);

		[Test]
		[ExpectedException(typeof (ArgumentNullException))]
		public void FileNameNull()
		{
			new FileConfigDataProvider(null);
		}

		[Test]
		[ExpectedException(typeof(ArgumentNullException))]
		public void FileNameEmpty()
		{
			new FileConfigDataProvider("");
		}

		[Test]
		[ExpectedException(typeof (FileNotFoundException))]
		public void FileNotExists()
		{
			new FileConfigDataProvider("X:/y/z");
		}

		[Test]
		public void ChangeNotification()
		{
			string fn = Path.Combine(_exeDir, "test.cfg");
			using (new StreamWriter(fn)) { }
			FileConfigDataProvider fcp = new FileConfigDataProvider(fn);
			bool changeFired = false;
			fcp.ConfigChanged += delegate { changeFired = true; };
			Assert.IsFalse(changeFired);
			using (new StreamWriter(fn)) { }
			// Allow fire notification
			Thread.Sleep(100);
			Assert.IsTrue(changeFired);
		}
	}
}
