using System;
using System.Reflection;

using NUnit.Framework;

namespace Rsdn.SmartApp
{
	using INamedSvc = IRegKeyedElementsService<string, TestNamedElementInfo>;

	[TestFixture]
	public class ExtensionManagerTest
	{
		private ServiceManager _svcManager;
		private ExtensionManager _extManager;

		[SetUp]
		protected void SetUp()
		{
			_svcManager = new ServiceManager(true);
			_extManager = new ExtensionManager(_svcManager);
		}

		[Test]
		public void SelfPublishing()
		{
			Assert.AreEqual(_extManager.GetService<IExtensionManager>(), _extManager);
		}

		[Test]
		public void AssemblyScanning()
		{
			_extManager.Scan(Assembly.GetExecutingAssembly().GetTypes(),
				new SimpleExtensionStrategy());
			Assert.AreEqual(SimpleExtensionStrategy.LastExtensionType,
				typeof (SimpleExtension));
		}

		[Test]
		[ExpectedException(typeof (ArgumentNullException))]
		public void NullTypes()
		{
			_extManager.Scan(null, null);
		}

		[Test]
		[ExpectedException(typeof(ArgumentNullException))]
		public void NullStrategy()
		{
			_extManager.Scan(new Type[0], null);
		}

		[Test]
		public void DisableAllScan()
		{
			_extManager.Scan(Assembly.GetExecutingAssembly().GetTypes(),
				new NamedElementStrategy(_svcManager));
			INamedSvc svc = _extManager.GetService<INamedSvc>();
			Assert.IsFalse(svc.ContainsElement(DisableAllExtension.Name));
		}

		[Test]
		public void DisableScanByParamType()
		{
			_extManager.Scan(Assembly.GetExecutingAssembly().GetTypes(),
				new NamedElementStrategy(_svcManager));
			INamedSvc svc = _extManager.GetService<INamedSvc>();
			Assert.IsFalse(svc.ContainsElement(DisableByParamTypeExtension.Name),
				"Disabled element registered");
			Assert.IsTrue(svc.ContainsElement(DisableByParamTypeExtension.Name2));
		}
	}
}
