using System;
using System.Reflection;

using NUnit.Framework;

namespace Rsdn.SmartApp
{
	using INamedSvc = IRegKeyedElementsService<string, TestNamedElementInfo>;
	using NamedSvc = RegKeyedElementsService<string, TestNamedElementInfo>;

	[TestFixture]
	public class RegistrationTest
	{
		private ServiceManager _svcManager;
		private ExtensionManager _extManager;

		[SetUp]
		protected void SetUp()
		{
			_svcManager = new ServiceManager();
			_extManager = new ExtensionManager(_svcManager);
		}

		[Test]
		public void RegElementsService()
		{
			RegElementsService<ElementInfo> svc = new RegElementsService<ElementInfo>();
			svc.Register(new ElementInfo(typeof (SampleElement)));
			TestRegElementsInService(svc);
		}

		private static void TestRegElementsInService<EI>(IRegElementsService<EI> svc)
			where EI : ElementInfo
		{
			EI[] elems = svc.GetRegisteredElements();
			Assert.IsNotNull(elems);
			EI sampleElem = null;
			foreach (EI elem in elems)
				if (elem.ElementType == typeof(SampleElement))
				{
					sampleElem = elem;
					break;
				}
			Assert.IsNotNull(sampleElem);
		}

		[Test]
		[ExpectedException(typeof (ArgumentNullException))]
		public void RegElementsServiceInfoIsNull()
		{
			RegElementsService<ElementInfo> svc = new RegElementsService<ElementInfo>();
			svc.Register(null);
		}

		[Test]
		public void RegElementsHelper()
		{
			_extManager.Scan(Assembly.GetExecutingAssembly().GetTypes(),
				new ElementStrategy(_svcManager));
			TestRegElements<ElementInfo>();
		}

		private void TestRegElements<EI>()
			where EI : ElementInfo
		{
			IRegElementsService<EI> svc =
				_extManager.GetService<IRegElementsService<EI>>();
			Assert.IsNotNull(svc);
			TestRegElementsInService<EI>(svc);
		}

		[Test]
		public void RegNamedElementsHelper()
		{
			_extManager.Scan(Assembly.GetExecutingAssembly().GetTypes(),
				new NamedElementStrategy(_svcManager));
			// Test old service inherited functionality
			TestRegElements<TestNamedElementInfo>();
			INamedSvc svc = _extManager.GetService<INamedSvc>();
			// Test new service inherited functionality
			TestRegElementsInService(svc);
			// Test own functionality
			Assert.IsFalse(svc.ContainsElement(""));
			Assert.IsTrue(svc.ContainsElement(SampleElement.Name));
			Assert.AreEqual(svc.GetElement(SampleElement.Name).ElementType,
				typeof (SampleElement));
		}

		[Test]
		[ExpectedException(typeof (ArgumentNullException))]
		public void NamedSvcContainsParamIsNull()
		{
			NamedSvc svc = new NamedSvc();
			svc.ContainsElement(null);
		}

		[Test]
		[ExpectedException(typeof(ArgumentNullException))]
		public void NamedSvcGetParamIsNull()
		{
			NamedSvc svc = new NamedSvc();
			svc.GetElement(null);
		}
	}
}
