﻿using NUnit.Framework;
using System;

namespace Rsdn.SmartApp
{
	[TestFixture]
	public class InstancingHelperTest
	{
		private ServiceManager _svcMgr;

		[SetUp]
		protected void SetUp()
		{
			_svcMgr = new ServiceManager();
			_svcMgr.Publish<ITestSvc>(new TestSvc());
		}

		[Test]
		[ExpectedException(typeof (ArgumentException))]
		public void NoPublicCtors()
		{
			InstancingHelper.CreateInstance<NoPublicCtors>(_svcMgr);
		}

		[Test]
		[ExpectedException(typeof (ArgumentException))]
		public void NoDefaultCtor()
		{
			InstancingHelper.CreateInstance<NoDefaultCtor>(_svcMgr);
		}

		[Test]
		public void DefaultCtorMarkup()
		{
			Assert.AreEqual("default",
				InstancingHelper.CreateInstance<MarkedDefaultCtor>(_svcMgr).Message);
		}

		[Test]
		[ExpectedException(typeof (ArgumentException))]
		public void MultipleDefaultCtors()
		{
			InstancingHelper.CreateInstance<MultipleDefaultCtors>(_svcMgr);
		}

		[Test]
		public void OnlyProviderParam()
		{
			Assert.AreEqual(_svcMgr, InstancingHelper.CreateInstance<OnlyProviderParam>(_svcMgr).Provider);
		}

		[Test]
		public void CustomParams()
		{
			var testMessage = "TestMessage";
			var c = InstancingHelper.CreateInstance<CustomParamsCtor>(_svcMgr,
				new InstancingCustomParam(typeof (string), testMessage));
			Assert.AreEqual(testMessage, c.Message);
		}

		#region TestSvc class
		private class TestSvc : ITestSvc
		{
			#region ITestSvc Members
			public string Message
			{
				get { return "TestSvc"; }
			}
			#endregion
		}
		#endregion
	}
}
